package net.awired.ajsl.core.system.implementation;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.awired.ajsl.core.system.enumeration.OsTypeList;
import net.awired.ajsl.core.system.interfaces.Os;
import net.awired.ajsl.core.system.interfaces.OsType;

public class OsManager<T extends OsType> {

    private static final Logger                     LOG            = Logger.getLogger(OsManager.class.getName());

    private static final String                     OS_NAME_KEY    = "os.name";
    private static final String                     OS_VERSION_KEY = "os.version";
    private static final String                     OS_ARCH_KEY    = "os.arch";

    private static final Class<? extends OsType<?>> OS_TYPE_ENUM   = OsTypeList.class;

    private T                                       osType;

    public OsManager(T osType) {
        this.osType = osType;
    }

    public Class<? extends Os> getOsClassInArray(List<Class<? extends Os>> classes) {
        for (Class<? extends Os> clazz : classes) {
            Class<T> o = osType.getOsInterface();
            if (o.isAssignableFrom(clazz)) {
                return clazz;
            }
        }
        return null;
    }

    public static OsManager<?> getInstance() {
        return getInstance(OS_TYPE_ENUM);
    }

    public static OsManager<?> getInstance(Class<? extends OsType<?>> osTypeEnum) {
        String name = System.getProperty(OS_NAME_KEY);
        String version = System.getProperty(OS_VERSION_KEY);
        OsType<?>[] constants = osTypeEnum.getEnumConstants();

        System.out.println("os name : " + name);
        System.out.println("os version : " + version);

        LOG.log(Level.FINER, "os name : " + name);
        LOG.log(Level.FINER, "os version : " + version);

        for (OsType<?> osType : constants) {
            if (osType.getOsName().equals(name)) {
                LOG.info("Current os is " + osType);
                return new OsManager<OsType<?>>(osType);
            }
        }
        return null;
    }

    // ////////////////////////////////////////////////////////////////////////

    public T getOsType() {
        return osType;
    }
}
